package org.implab.gradle;

import java.io.ByteArrayOutputStream;
import java.io.Closeable;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;
import java.util.Scanner;

import org.gradle.api.Action;
import groovy.json.JsonGenerator;
import groovy.json.JsonOutput;
import groovy.json.JsonGenerator.Converter;
import groovy.lang.Closure;

public final class Utils {
    public static void redirectIO(final InputStream src, final Action<String> consumer) {
        new Thread(() -> {
            try (Scanner sc = new Scanner(src)) {
                while (sc.hasNextLine()) {
                    consumer.execute(sc.nextLine());
                }
            }
        }).start();
    }

    public static String execCmd(List<String> args, String charset) {
        if (args == null || args.size() == 0)
            throw new IllegalArgumentException();

        ProcessBuilder builder = new ProcessBuilder(args);

        Process p;
        try {
            p = builder.start();

            String output = readAll(p.getInputStream(), charset);

            int code = p.waitFor();
            if (code != 0)
                throw new RuntimeException(String.format("The process `%s` failed with code: %s", args.get(0), code));

            return output;
        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(String.format("Failed to execute `%s`", args.get(0)), e);
        }

    }

    public static void redirectIO(final InputStream src, final File file) {
        new Thread(() -> {
            try (OutputStream out = new FileOutputStream(file)) {
                src.transferTo(out);
            } catch (Exception e) {
                // silence!
            }
        }).start();
    }

    public static String readAll(final InputStream src) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        src.transferTo(out);
        return out.toString();
    }

    public static String readAll(final InputStream src, String charset) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        src.transferTo(out);
        return out.toString(charset);
    }

    public static JsonGenerator createDefaultJsonGenerator() {
        return new JsonGenerator.Options()
                .excludeNulls()
                .addConverter(new Converter() {
                    public boolean handles(Class<?> type) {
                        return (File.class == type);
                    }

                    public Object convert(Object value, String key) {
                        return ((File) value).getPath();
                    }
                })
                .build();
    }

    public static void closeSilent(Closeable handle) {
        try {
            handle.close();
        } catch (Exception e) {
            // silence!
        }
    }

    public static String toJsonPretty(Object value) {
        return JsonOutput.prettyPrint(createDefaultJsonGenerator().toJson(value));
    }

    public static boolean isNullOrEmptyString(String value) {
        return (value == null || value.length() == 0);
    }

    public static <T> Action<T> wrapClosure(Closure<?> closure) {
        return x -> {
            closure.setDelegate(x);
            closure.setResolveStrategy(Closure.DELEGATE_FIRST);
            closure.call(x);
        };
    }

}
